@@ -1,5 +1,5 @@
 
-#include <private/kspimpl.h>             /*I "petscksp.h" I*/
+#include <petsc/private/kspimpl.h>             /*I "petscksp.h" I*/
 #include <../src/ksp/ksp/impls/cg/nash/nashimpl.h>
 
 #define NASH_PRECONDITIONED_DIRECTION   0
@@ -32,7 +32,7 @@ PetscErrorCode  KSPNASHSetRadius(KSP ksp, PetscReal radius)
 
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ksp, KSP_CLASSID, 1);
-  if (radius < 0.0) SETERRQ(((PetscObject)ksp)->comm,PETSC_ERR_ARG_OUTOFRANGE, "Radius negative");
+  if (radius < 0.0) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE, "Radius negative");
   PetscValidLogicalCollectiveReal(ksp,radius,2);
   ierr = PetscTryMethod(ksp,"KSPNASHSetRadius_C",(KSP,PetscReal),(ksp,radius));CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -93,23 +93,22 @@ PetscErrorCode  KSPNASHGetObjFcn(KSP ksp, PetscReal *o_fcn)
 
 PetscErrorCode KSPSolve_NASH(KSP ksp)
 {
-#ifdef PETSC_USE_COMPLEX
-  SETERRQ(((PetscObject)ksp)->comm,PETSC_ERR_SUP, "NASH is not available for complex systems");
+#if defined(PETSC_USE_COMPLEX)
+  SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP, "NASH is not available for complex systems");
 #else
-  KSP_NASH       *cg = (KSP_NASH *)ksp->data;
+  KSP_NASH       *cg = (KSP_NASH*)ksp->data;
   PetscErrorCode ierr;
-  MatStructure   pflag;
   Mat            Qmat, Mmat;
   Vec            r, z, p, d;
   PC             pc;
 
-  PetscReal      norm_r, norm_d, norm_dp1, norm_p, dMp;
-  PetscReal      alpha, beta, kappa, rz, rzm1;
-  PetscReal      rr, r2, step;
+  PetscReal norm_r, norm_d, norm_dp1, norm_p, dMp;
+  PetscReal alpha, beta, kappa, rz, rzm1;
+  PetscReal rr, r2, step;
 
-  PetscInt       max_cg_its;
+  PetscInt max_cg_its;
 
-  PetscBool      diagonalscale;
+  PetscBool diagonalscale;
 
   PetscFunctionBegin;
   /***************************************************************************/
@@ -117,8 +116,8 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /***************************************************************************/
 
   ierr = PCGetDiagonalScale(ksp->pc, &diagonalscale);CHKERRQ(ierr);
-  if (diagonalscale) SETERRQ1(((PetscObject)ksp)->comm,PETSC_ERR_SUP, "Krylov method %s does not support diagonal scaling", ((PetscObject)ksp)->type_name);
-  if (cg->radius < 0.0) SETERRQ(((PetscObject)ksp)->comm,PETSC_ERR_ARG_OUTOFRANGE, "Input error: radius < 0");
+  if (diagonalscale) SETERRQ1(PetscObjectComm((PetscObject)ksp),PETSC_ERR_SUP, "Krylov method %s does not support diagonal scaling", ((PetscObject)ksp)->type_name);
+  if (cg->radius < 0.0) SETERRQ(PetscObjectComm((PetscObject)ksp),PETSC_ERR_ARG_OUTOFRANGE, "Input error: radius < 0");
 
   /***************************************************************************/
   /* Get the workspace vectors and initialize variables                      */
@@ -131,11 +130,11 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   d  = ksp->vec_sol;
   pc = ksp->pc;
 
-  ierr = PCGetOperators(pc, &Qmat, &Mmat, &pflag);CHKERRQ(ierr);
+  ierr = PCGetOperators(pc, &Qmat, &Mmat);CHKERRQ(ierr);
 
-  ierr = VecGetSize(d, &max_cg_its);CHKERRQ(ierr);
+  ierr       = VecGetSize(d, &max_cg_its);CHKERRQ(ierr);
   max_cg_its = PetscMin(max_cg_its, ksp->max_it);
-  ksp->its = 0;
+  ksp->its   = 0;
 
   /***************************************************************************/
   /* Initialize objective function and direction.                            */
@@ -143,7 +142,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
 
   cg->o_fcn = 0.0;
 
-  ierr = VecSet(d, 0.0);CHKERRQ(ierr);			/* d = 0             */
+  ierr       = VecSet(d, 0.0);CHKERRQ(ierr);            /* d = 0             */
   cg->norm_d = 0.0;
 
   /***************************************************************************/
@@ -152,18 +151,9 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* need be performed only once.                                            */
   /***************************************************************************/
 
-  ierr = VecCopy(ksp->vec_rhs, r);CHKERRQ(ierr);	/* r = -grad         */
-  ierr = VecDot(r, r, &rr);CHKERRQ(ierr);		/* rr = r^T r        */
-  if (PetscIsInfOrNanScalar(rr)) {
-    /*************************************************************************/
-    /* The right-hand side contains not-a-number or an infinite value.       */
-    /* The gradient step does not work; return a zero value for the step.    */
-    /*************************************************************************/
-
-    ksp->reason = KSP_DIVERGED_NAN;
-    ierr = PetscInfo1(ksp, "KSPSolve_NASH: bad right-hand side: rr=%g\n", rr);CHKERRQ(ierr);
-    PetscFunctionReturn(0);
-  }
+  ierr = VecCopy(ksp->vec_rhs, r);CHKERRQ(ierr);        /* r = -grad         */
+  ierr = VecDot(r, r, &rr);CHKERRQ(ierr);               /* rr = r^T r        */
+  KSPCheckDot(ksp,rr);
 
   /***************************************************************************/
   /* Check the preconditioner for numerical problems and for positive        */
@@ -171,36 +161,35 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* performed only once.                                                    */
   /***************************************************************************/
 
-  ierr = KSP_PCApply(ksp, r, z);CHKERRQ(ierr);		/* z = inv(M) r      */
-  ierr = VecDot(r, z, &rz);CHKERRQ(ierr);		/* rz = r^T inv(M) r */
+  ierr = KSP_PCApply(ksp, r, z);CHKERRQ(ierr);          /* z = inv(M) r      */
+  ierr = VecDot(r, z, &rz);CHKERRQ(ierr);               /* rz = r^T inv(M) r */
   if (PetscIsInfOrNanScalar(rz)) {
     /*************************************************************************/
     /* The preconditioner contains not-a-number or an infinite value.        */
     /* Return the gradient direction intersected with the trust region.      */
     /*************************************************************************/
 
-    ksp->reason = KSP_DIVERGED_NAN;
-    ierr = PetscInfo1(ksp, "KSPSolve_NASH: bad preconditioner: rz=%g\n", rz);CHKERRQ(ierr);
+    ksp->reason = KSP_DIVERGED_NANORINF;
+    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: bad preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
-        alpha = 1.0;
+        alpha      = 1.0;
         cg->norm_d = PetscSqrtReal(rr);
-      }
-      else {
-        alpha = PetscSqrtReal(r2 / rr);
+      } else {
+        alpha      = PetscSqrtReal(r2 / rr);
         cg->norm_d = cg->radius;
       }
 
-      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);	/* d = d + alpha r   */
+      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);        /* d = d + alpha r   */
 
       /***********************************************************************/
       /* Compute objective function.                                         */
       /***********************************************************************/
 
-      ierr = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
-      ierr = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
-      ierr = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
+      ierr      = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
+      ierr      = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
+      ierr      = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
       cg->o_fcn = -cg->o_fcn;
       ++ksp->its;
     }
@@ -216,27 +205,26 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
-    ierr = PetscInfo1(ksp, "KSPSolve_NASH: indefinite preconditioner: rz=%g\n", rz);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
-        alpha = 1.0;
+        alpha      = 1.0;
         cg->norm_d = PetscSqrtReal(rr);
-      }
-      else {
-        alpha = PetscSqrtReal(r2 / rr);
+      } else {
+        alpha      = PetscSqrtReal(r2 / rr);
         cg->norm_d = cg->radius;
       }
 
-      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);	/* d = d + alpha r   */
+      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);        /* d = d + alpha r   */
 
       /***********************************************************************/
       /* Compute objective function.                                         */
       /***********************************************************************/
 
-      ierr = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
-      ierr = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
-      ierr = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
+      ierr      = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
+      ierr      = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
+      ierr      = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
       cg->o_fcn = -cg->o_fcn;
       ++ksp->its;
     }
@@ -250,17 +238,17 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* gradient iteration has been performed.                                  */
   /***************************************************************************/
 
-  switch(ksp->normtype) {
+  switch (ksp->normtype) {
   case KSP_NORM_PRECONDITIONED:
-    ierr = VecNorm(z, NORM_2, &norm_r);CHKERRQ(ierr);	/* norm_r = |z|      */
+    ierr = VecNorm(z, NORM_2, &norm_r);CHKERRQ(ierr);   /* norm_r = |z|      */
     break;
 
   case KSP_NORM_UNPRECONDITIONED:
-    norm_r = PetscSqrtReal(rr);					/* norm_r = |r|      */
+    norm_r = PetscSqrtReal(rr);                                 /* norm_r = |r|      */
     break;
 
   case KSP_NORM_NATURAL:
-    norm_r = PetscSqrtReal(rz);					/* norm_r = |r|_M    */
+    norm_r = PetscSqrtReal(rz);                                 /* norm_r = |r|_M    */
     break;
 
   default:
@@ -268,8 +256,8 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     break;
   }
 
-  KSPLogResidualHistory(ksp, norm_r);
-  ierr = KSPMonitor(ksp, ksp->its, norm_r);CHKERRQ(ierr);
+  ierr       = KSPLogResidualHistory(ksp, norm_r);CHKERRQ(ierr);
+  ierr       = KSPMonitor(ksp, ksp->its, norm_r);CHKERRQ(ierr);
   ksp->rnorm = norm_r;
 
   ierr = (*ksp->converged)(ksp, ksp->its, norm_r, &ksp->reason, ksp->cnvP);CHKERRQ(ierr);
@@ -278,15 +266,15 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* Compute the first direction and update the iteration.                   */
   /***************************************************************************/
 
-  ierr = VecCopy(z, p);CHKERRQ(ierr);			/* p = z             */
-  ierr = KSP_MatMult(ksp, Qmat, p, z);CHKERRQ(ierr);	/* z = Q * p         */
+  ierr = VecCopy(z, p);CHKERRQ(ierr);                   /* p = z             */
+  ierr = KSP_MatMult(ksp, Qmat, p, z);CHKERRQ(ierr);    /* z = Q * p         */
   ++ksp->its;
 
   /***************************************************************************/
   /* Check the matrix for numerical problems.                                */
   /***************************************************************************/
 
-  ierr = VecDot(p, z, &kappa);CHKERRQ(ierr);		/* kappa = p^T Q p   */
+  ierr = VecDot(p, z, &kappa);CHKERRQ(ierr);            /* kappa = p^T Q p   */
   if (PetscIsInfOrNanScalar(kappa)) {
     /*************************************************************************/
     /* The matrix produced not-a-number or an infinite value.  In this case, */
@@ -294,28 +282,27 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /* only be checked once.                                                 */
     /*************************************************************************/
 
-    ksp->reason = KSP_DIVERGED_NAN;
-    ierr = PetscInfo1(ksp, "KSPSolve_NASH: bad matrix: kappa=%g\n", kappa);CHKERRQ(ierr);
+    ksp->reason = KSP_DIVERGED_NANORINF;
+    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: bad matrix: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
 
     if (cg->radius) {
       if (r2 >= rr) {
-        alpha = 1.0;
+        alpha      = 1.0;
         cg->norm_d = PetscSqrtReal(rr);
-      }
-      else {
-        alpha = PetscSqrtReal(r2 / rr);
+      } else {
+        alpha      = PetscSqrtReal(r2 / rr);
         cg->norm_d = cg->radius;
       }
 
-      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);	/* d = d + alpha r   */
+      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);        /* d = d + alpha r   */
 
       /***********************************************************************/
       /* Compute objective function.                                         */
       /***********************************************************************/
 
-      ierr = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
-      ierr = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
-      ierr = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
+      ierr      = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
+      ierr      = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
+      ierr      = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
       cg->o_fcn = -cg->o_fcn;
       ++ksp->its;
     }
@@ -326,9 +313,9 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* Initialize variables for calculating the norm of the direction.         */
   /***************************************************************************/
 
-  dMp = 0.0;
+  dMp    = 0.0;
   norm_d = 0.0;
-  switch(cg->dtype) {
+  switch (cg->dtype) {
   case NASH_PRECONDITIONED_DIRECTION:
     norm_p = rz;
     break;
@@ -350,7 +337,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     ksp->reason = KSP_CONVERGED_CG_NEG_CURVE;
-    ierr = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", kappa);CHKERRQ(ierr);
+    ierr        = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
 
     if (cg->radius && norm_p > 0.0) {
       /***********************************************************************/
@@ -358,41 +345,39 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /* trust region.                                                       */
       /***********************************************************************/
 
-      step = PetscSqrtReal(r2 / norm_p);
+      step       = PetscSqrtReal(r2 / norm_p);
       cg->norm_d = cg->radius;
 
-      ierr = VecAXPY(d, step, p);CHKERRQ(ierr);	/* d = d + step p    */
+      ierr = VecAXPY(d, step, p);CHKERRQ(ierr); /* d = d + step p    */
 
       /***********************************************************************/
       /* Update objective function.                                          */
       /***********************************************************************/
 
       cg->o_fcn += step * (0.5 * step * kappa - rz);
-    }
-    else if (cg->radius) {
+    } else if (cg->radius) {
       /***********************************************************************/
       /* The norm of the preconditioned direction is zero; use the gradient  */
       /* step.                                                               */
       /***********************************************************************/
 
       if (r2 >= rr) {
-        alpha = 1.0;
+        alpha      = 1.0;
         cg->norm_d = PetscSqrtReal(rr);
-      }
-      else {
-        alpha = PetscSqrtReal(r2 / rr);
+      } else {
+        alpha      = PetscSqrtReal(r2 / rr);
         cg->norm_d = cg->radius;
       }
 
-      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);	/* d = d + alpha r   */
+      ierr = VecAXPY(d, alpha, r);CHKERRQ(ierr);        /* d = d + alpha r   */
 
       /***********************************************************************/
       /* Compute objective function.                                         */
       /***********************************************************************/
 
-      ierr = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
-      ierr = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
-      ierr = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
+      ierr      = KSP_MatMult(ksp, Qmat, d, z);CHKERRQ(ierr);
+      ierr      = VecAYPX(z, -0.5, ksp->vec_rhs);CHKERRQ(ierr);
+      ierr      = VecDot(d, z, &cg->o_fcn);CHKERRQ(ierr);
       cg->o_fcn = -cg->o_fcn;
       ++ksp->its;
     }
@@ -405,7 +390,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
   /* gradient method breaks down.                                            */
   /***************************************************************************/
 
-  while(1) {
+  while (1) {
     /*************************************************************************/
     /* Know that kappa is nonzero, because we have not broken down, so we    */
     /* can compute the steplength.                                           */
@@ -426,25 +411,24 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_CONVERGED_CG_CONSTRAINED;
-      ierr = PetscInfo1(ksp, "KSPSolve_NASH: constrained step: radius=%g\n", cg->radius);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: constrained step: radius=%g\n", (double)cg->radius);CHKERRQ(ierr);
 
       if (norm_p > 0.0) {
-	/*********************************************************************/
-	/* Follow the direction to the boundary of the trust region.         */
-	/*********************************************************************/
+        /*********************************************************************/
+        /* Follow the direction to the boundary of the trust region.         */
+        /*********************************************************************/
 
-        step = (PetscSqrtReal(dMp*dMp+norm_p*(r2-norm_d))-dMp)/norm_p;
+        step       = (PetscSqrtReal(dMp*dMp+norm_p*(r2-norm_d))-dMp)/norm_p;
         cg->norm_d = cg->radius;
 
-        ierr = VecAXPY(d, step, p);CHKERRQ(ierr);	/* d = d + step p    */
+        ierr = VecAXPY(d, step, p);CHKERRQ(ierr);       /* d = d + step p    */
 
         /*********************************************************************/
         /* Update objective function.                                        */
         /*********************************************************************/
 
         cg->o_fcn += step * (0.5 * step * kappa - rz);
-      }
-      else {
+      } else {
         /*********************************************************************/
         /* The norm of the direction is zero; there is nothing to follow.    */
         /*********************************************************************/
@@ -456,11 +440,11 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /* Now we can update the direction and residual.                         */
     /*************************************************************************/
 
-    ierr = VecAXPY(d, alpha, p);CHKERRQ(ierr);		/* d = d + alpha p   */
-    ierr = VecAXPY(r, -alpha, z);			/* r = r - alpha Q p */
-    ierr = KSP_PCApply(ksp, r, z);CHKERRQ(ierr);	/* z = inv(M) r      */
+    ierr = VecAXPY(d, alpha, p);CHKERRQ(ierr);          /* d = d + alpha p   */
+    ierr = VecAXPY(r, -alpha, z);CHKERRQ(ierr);         /* r = r - alpha Q p */
+    ierr = KSP_PCApply(ksp, r, z);CHKERRQ(ierr);        /* z = inv(M) r      */
 
-    switch(cg->dtype) {
+    switch (cg->dtype) {
     case NASH_PRECONDITIONED_DIRECTION:
       norm_d = norm_dp1;
       break;
@@ -482,14 +466,14 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     rzm1 = rz;
-    ierr = VecDot(r, z, &rz);CHKERRQ(ierr);		/* rz = r^T z        */
+    ierr = VecDot(r, z, &rz);CHKERRQ(ierr);             /* rz = r^T z        */
     if (rz < 0.0) {
       /***********************************************************************/
       /* The preconditioner is indefinite.                                   */
       /***********************************************************************/
 
       ksp->reason = KSP_DIVERGED_INDEFINITE_PC;
-      ierr = PetscInfo1(ksp, "KSPSolve_NASH: cg indefinite preconditioner: rz=%g\n", rz);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: cg indefinite preconditioner: rz=%g\n", (double)rz);CHKERRQ(ierr);
       break;
     }
 
@@ -498,17 +482,17 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /* Compute the residual and check for convergence.                       */
     /*************************************************************************/
 
-    switch(ksp->normtype) {
+    switch (ksp->normtype) {
     case KSP_NORM_PRECONDITIONED:
-      ierr = VecNorm(z, NORM_2, &norm_r);CHKERRQ(ierr);/* norm_r = |z|      */
+      ierr = VecNorm(z, NORM_2, &norm_r);CHKERRQ(ierr); /* norm_r = |z|      */
       break;
 
     case KSP_NORM_UNPRECONDITIONED:
-      ierr = VecNorm(r, NORM_2, &norm_r);CHKERRQ(ierr);/* norm_r = |r|      */
+      ierr = VecNorm(r, NORM_2, &norm_r);CHKERRQ(ierr); /* norm_r = |r|      */
       break;
 
     case KSP_NORM_NATURAL:
-      norm_r = PetscSqrtReal(rz);				/* norm_r = |r|_M    */
+      norm_r = PetscSqrtReal(rz);                               /* norm_r = |r|_M    */
       break;
 
     default:
@@ -516,17 +500,17 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       break;
     }
 
-    KSPLogResidualHistory(ksp, norm_r);
-    ierr = KSPMonitor(ksp, ksp->its, norm_r);CHKERRQ(ierr);
+    ierr       = KSPLogResidualHistory(ksp, norm_r);CHKERRQ(ierr);
+    ierr       = KSPMonitor(ksp, ksp->its, norm_r);CHKERRQ(ierr);
     ksp->rnorm = norm_r;
-  
+
     ierr = (*ksp->converged)(ksp, ksp->its, norm_r, &ksp->reason, ksp->cnvP);CHKERRQ(ierr);
     if (ksp->reason) {
       /***********************************************************************/
       /* The method has converged.                                           */
       /***********************************************************************/
 
-      ierr = PetscInfo2(ksp, "KSPSolve_NASH: truncated step: rnorm=%g, radius=%g\n", norm_r, cg->radius);CHKERRQ(ierr);
+      ierr = PetscInfo2(ksp, "KSPSolve_NASH: truncated step: rnorm=%g, radius=%g\n", (double)norm_r, (double)cg->radius);CHKERRQ(ierr);
       break;
     }
 
@@ -535,13 +519,13 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /*************************************************************************/
 
     beta = rz / rzm1;
-    if (fabs(beta) <= 0.0) {
+    if (PetscAbsReal(beta) <= 0.0) {
       /***********************************************************************/
       /* Conjugate gradients has broken down.                                */
       /***********************************************************************/
 
       ksp->reason = KSP_DIVERGED_BREAKDOWN;
-      ierr = PetscInfo1(ksp, "KSPSolve_NASH: breakdown: beta=%g\n", beta);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: breakdown: beta=%g\n", (double)beta);CHKERRQ(ierr);
       break;
     }
 
@@ -551,7 +535,7 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
 
     if (ksp->its >= max_cg_its) {
       ksp->reason = KSP_DIVERGED_ITS;
-      ierr = PetscInfo1(ksp, "KSPSolve_NASH: iterlim: its=%d\n", ksp->its);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: iterlim: its=%D\n", ksp->its);CHKERRQ(ierr);
       break;
     }
 
@@ -561,9 +545,9 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
 
     ierr = VecAYPX(p, beta, z);CHKERRQ(ierr);          /* p = z + beta p    */
 
-    switch(cg->dtype) {
+    switch (cg->dtype) {
     case NASH_PRECONDITIONED_DIRECTION:
-      dMp = beta*(dMp + alpha*norm_p);
+      dMp    = beta*(dMp + alpha*norm_p);
       norm_p = beta*(rzm1 + beta*norm_p);
       break;
 
@@ -577,8 +561,8 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
     /* Compute the new direction and update the iteration.                   */
     /*************************************************************************/
 
-    ierr = KSP_MatMult(ksp, Qmat, p, z);CHKERRQ(ierr);	/* z = Q * p         */
-    ierr = VecDot(p, z, &kappa);CHKERRQ(ierr);		/* kappa = p^T Q p   */
+    ierr = KSP_MatMult(ksp, Qmat, p, z);CHKERRQ(ierr);  /* z = Q * p         */
+    ierr = VecDot(p, z, &kappa);CHKERRQ(ierr);          /* kappa = p^T Q p   */
     ++ksp->its;
 
     /*************************************************************************/
@@ -592,11 +576,10 @@ PetscErrorCode KSPSolve_NASH(KSP ksp)
       /***********************************************************************/
 
       ksp->reason = KSP_CONVERGED_CG_NEG_CURVE;
-      ierr = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", kappa);CHKERRQ(ierr);
+      ierr        = PetscInfo1(ksp, "KSPSolve_NASH: negative curvature: kappa=%g\n", (double)kappa);CHKERRQ(ierr);
       break;
     }
   }
-
   PetscFunctionReturn(0);
 #endif
 }
@@ -612,7 +595,7 @@ PetscErrorCode KSPSetUp_NASH(KSP ksp)
   /* Set work vectors needed by conjugate gradient method and allocate       */
   /***************************************************************************/
 
-  ierr = KSPDefaultGetWork(ksp, 3);CHKERRQ(ierr);
+  ierr = KSPSetWorkVecs(ksp, 3);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -623,28 +606,27 @@ PetscErrorCode KSPDestroy_NASH(KSP ksp)
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
- /***************************************************************************/
+  /***************************************************************************/
   /* Clear composed functions                                                */
   /***************************************************************************/
 
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,"KSPNASHSetRadius_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,"KSPNASHGetNormD_C","",PETSC_NULL);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,"KSPNASHGetObjFcn_C","",PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHSetRadius_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetNormD_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetObjFcn_C",NULL);CHKERRQ(ierr);
 
   /***************************************************************************/
   /* Destroy KSP object.                                                     */
   /***************************************************************************/
 
-  ierr = KSPDefaultDestroy(ksp);CHKERRQ(ierr);
+  ierr = KSPDestroyDefault(ksp);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "KSPNASHSetRadius_NASH"
-PetscErrorCode  KSPNASHSetRadius_NASH(KSP ksp, PetscReal radius)
+static PetscErrorCode  KSPNASHSetRadius_NASH(KSP ksp, PetscReal radius)
 {
-  KSP_NASH *cg = (KSP_NASH *)ksp->data;
+  KSP_NASH *cg = (KSP_NASH*)ksp->data;
 
   PetscFunctionBegin;
   cg->radius = radius;
@@ -653,9 +635,9 @@ PetscErrorCode  KSPNASHSetRadius_NASH(KSP ksp, PetscReal radius)
 
 #undef __FUNCT__
 #define __FUNCT__ "KSPNASHGetNormD_NASH"
-PetscErrorCode  KSPNASHGetNormD_NASH(KSP ksp, PetscReal *norm_d)
+static PetscErrorCode  KSPNASHGetNormD_NASH(KSP ksp, PetscReal *norm_d)
 {
-  KSP_NASH *cg = (KSP_NASH *)ksp->data;
+  KSP_NASH *cg = (KSP_NASH*)ksp->data;
 
   PetscFunctionBegin;
   *norm_d = cg->norm_d;
@@ -664,28 +646,28 @@ PetscErrorCode  KSPNASHGetNormD_NASH(KSP ksp, PetscReal *norm_d)
 
 #undef __FUNCT__
 #define __FUNCT__ "KSPNASHGetObjFcn_NASH"
-PetscErrorCode  KSPNASHGetObjFcn_NASH(KSP ksp, PetscReal *o_fcn){
-  KSP_NASH *cg = (KSP_NASH *)ksp->data;
+static PetscErrorCode  KSPNASHGetObjFcn_NASH(KSP ksp, PetscReal *o_fcn)
+{
+  KSP_NASH *cg = (KSP_NASH*)ksp->data;
 
   PetscFunctionBegin;
   *o_fcn = cg->o_fcn;
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
 
 #undef __FUNCT__
 #define __FUNCT__ "KSPSetFromOptions_NASH"
-PetscErrorCode KSPSetFromOptions_NASH(KSP ksp)
+PetscErrorCode KSPSetFromOptions_NASH(PetscOptionItems *PetscOptionsObject,KSP ksp)
 {
   PetscErrorCode ierr;
-  KSP_NASH       *cg = (KSP_NASH *)ksp->data;
+  KSP_NASH       *cg = (KSP_NASH*)ksp->data;
 
   PetscFunctionBegin;
-  ierr = PetscOptionsHead("KSP NASH options");CHKERRQ(ierr);
+  ierr = PetscOptionsHead(PetscOptionsObject,"KSP NASH options");CHKERRQ(ierr);
 
-  ierr = PetscOptionsReal("-ksp_nash_radius", "Trust Region Radius", "KSPNASHSetRadius", cg->radius, &cg->radius, PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsReal("-ksp_nash_radius", "Trust Region Radius", "KSPNASHSetRadius", cg->radius, &cg->radius, NULL);CHKERRQ(ierr);
 
-  ierr = PetscOptionsEList("-ksp_nash_dtype", "Norm used for direction", "", DType_Table, NASH_DIRECTION_TYPES, DType_Table[cg->dtype], &cg->dtype, PETSC_NULL);CHKERRQ(ierr);
+  ierr = PetscOptionsEList("-ksp_nash_dtype", "Norm used for direction", "", DType_Table, NASH_DIRECTION_TYPES, DType_Table[cg->dtype], &cg->dtype, NULL);CHKERRQ(ierr);
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -694,7 +676,7 @@ PetscErrorCode KSPSetFromOptions_NASH(KSP ksp)
 /*MC
      KSPNASH -   Code to run conjugate gradient method subject to a constraint
          on the solution norm. This is used in Trust Region methods for
-         nonlinear equations, SNESTR
+         nonlinear equations, SNESNEWTONTR
 
    Options Database Keys:
 .      -ksp_nash_radius <r> - Trust Region Radius
@@ -715,7 +697,7 @@ PetscErrorCode KSPSetFromOptions_NASH(KSP ksp)
    where
 
      delta is the trust region radius,
-     g is the gradient vector, 
+     g is the gradient vector,
      H is the Hessian approximation, and
      M is the positive definite preconditioner matrix.
 
@@ -730,49 +712,38 @@ $  other KSP converged/diverged reasons
 .seealso:  KSPCreate(), KSPSetType(), KSPType (for list of available types), KSP, KSPNASHSetRadius(), KSPNASHGetNormD(), KSPNASHGetObjFcn()
 M*/
 
-EXTERN_C_BEGIN
 #undef __FUNCT__
 #define __FUNCT__ "KSPCreate_NASH"
-PetscErrorCode  KSPCreate_NASH(KSP ksp)
+PETSC_EXTERN PetscErrorCode KSPCreate_NASH(KSP ksp)
 {
   PetscErrorCode ierr;
   KSP_NASH       *cg;
 
   PetscFunctionBegin;
-  ierr = PetscNewLog(ksp,KSP_NASH, &cg);CHKERRQ(ierr);
+  ierr       = PetscNewLog(ksp,&cg);CHKERRQ(ierr);
   cg->radius = 0.0;
-  cg->dtype = NASH_UNPRECONDITIONED_DIRECTION;
+  cg->dtype  = NASH_UNPRECONDITIONED_DIRECTION;
 
-  ksp->data = (void *) cg;
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,1);CHKERRQ(ierr);
-  ierr = KSPSetSupportedNorm(ksp,KSP_NORM_NATURAL,PC_LEFT,1);CHKERRQ(ierr);
+  ksp->data = (void*) cg;
+  ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_UNPRECONDITIONED,PC_LEFT,3);CHKERRQ(ierr);
+  ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_PRECONDITIONED,PC_LEFT,2);CHKERRQ(ierr);
+  ierr      = KSPSetSupportedNorm(ksp,KSP_NORM_NATURAL,PC_LEFT,2);CHKERRQ(ierr);
 
   /***************************************************************************/
   /* Sets the functions that are associated with this data structure         */
   /* (in C++ this is the same as defining virtual functions).                */
   /***************************************************************************/
 
-  ksp->ops->setup                = KSPSetUp_NASH;
-  ksp->ops->solve                = KSPSolve_NASH;
-  ksp->ops->destroy              = KSPDestroy_NASH;
-  ksp->ops->setfromoptions       = KSPSetFromOptions_NASH;
-  ksp->ops->buildsolution        = KSPDefaultBuildSolution;
-  ksp->ops->buildresidual        = KSPDefaultBuildResidual;
-  ksp->ops->view                 = 0;
+  ksp->ops->setup          = KSPSetUp_NASH;
+  ksp->ops->solve          = KSPSolve_NASH;
+  ksp->ops->destroy        = KSPDestroy_NASH;
+  ksp->ops->setfromoptions = KSPSetFromOptions_NASH;
+  ksp->ops->buildsolution  = KSPBuildSolutionDefault;
+  ksp->ops->buildresidual  = KSPBuildResidualDefault;
+  ksp->ops->view           = 0;
 
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,
-				           "KSPNASHSetRadius_C",
-                                           "KSPNASHSetRadius_NASH",
-                                            KSPNASHSetRadius_NASH);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,
-				           "KSPNASHGetNormD_C",
-                                           "KSPNASHGetNormD_NASH",
-                                            KSPNASHGetNormD_NASH);CHKERRQ(ierr);
-  ierr = PetscObjectComposeFunctionDynamic((PetscObject)ksp,
-                                           "KSPNASHGetObjFcn_C",
-                                           "KSPNASHGetObjFcn_NASH",
-                                            KSPNASHGetObjFcn_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHSetRadius_C",KSPNASHSetRadius_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetNormD_C",KSPNASHGetNormD_NASH);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)ksp,"KSPNASHGetObjFcn_C",KSPNASHGetObjFcn_NASH);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
-EXTERN_C_END
