@@ -1,5 +1,5 @@
 
-/* 
+/*
    This file contains simple binary read/write routines.
  */
 
@@ -9,13 +9,15 @@
 #if defined(PETSC_HAVE_UNISTD_H)
 #include <unistd.h>
 #endif
-#if defined (PETSC_HAVE_IO_H)
+#if defined(PETSC_HAVE_IO_H)
 #include <io.h>
 #endif
 #include <petscbt.h>
 
+const char *const PetscFileModes[] = {"READ","WRITE","APPEND","UPDATE","APPEND_UPDATE","PetscFileMode","PETSC_FILE_",0};
+
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapEnum"
 /*
   PetscByteSwapEnum - Swap bytes in a  PETSc Enum
@@ -23,24 +25,20 @@
 */
 PetscErrorCode  PetscByteSwapEnum(PetscEnum *buff,PetscInt n)
 {
-  PetscInt   i,j;
-  PetscEnum   tmp = ENUM_DUMMY;
-  char       *ptr1,*ptr2 = (char*)&tmp;
-                                   
+  PetscInt  i,j;
+  PetscEnum tmp = ENUM_DUMMY;
+  char      *ptr1,*ptr2 = (char*)&tmp;
+
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff + j);
-    for (i=0; i<(PetscInt)sizeof(PetscEnum); i++) {
-      ptr2[i] = ptr1[sizeof(PetscEnum)-1-i];
-    }
-    for (i=0; i<(PetscInt)sizeof(PetscEnum); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt)sizeof(PetscEnum); i++) ptr2[i] = ptr1[sizeof(PetscEnum)-1-i];
+    for (i=0; i<(PetscInt)sizeof(PetscEnum); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapBool"
 /*
   PetscByteSwapBool - Swap bytes in a  PETSc Bool
@@ -48,72 +46,60 @@ PetscErrorCode  PetscByteSwapEnum(PetscEnum *buff,PetscInt n)
 */
 PetscErrorCode  PetscByteSwapBool(PetscBool *buff,PetscInt n)
 {
-  PetscInt    i,j;
-  PetscBool   tmp = PETSC_FALSE;
-  char        *ptr1,*ptr2 = (char*)&tmp;
-                                   
+  PetscInt  i,j;
+  PetscBool tmp = PETSC_FALSE;
+  char      *ptr1,*ptr2 = (char*)&tmp;
+
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff + j);
-    for (i=0; i<(PetscInt)sizeof(PetscBool); i++) {
-      ptr2[i] = ptr1[sizeof(PetscBool)-1-i];
-    }
-    for (i=0; i<(PetscInt)sizeof(PetscBool); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt)sizeof(PetscBool); i++) ptr2[i] = ptr1[sizeof(PetscBool)-1-i];
+    for (i=0; i<(PetscInt)sizeof(PetscBool); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapInt"
 /*
-  PetscByteSwapInt - Swap bytes in a  PETSc integer (which may be 32 or 64 bits) 
+  PetscByteSwapInt - Swap bytes in a  PETSc integer (which may be 32 or 64 bits)
 
 */
 PetscErrorCode  PetscByteSwapInt(PetscInt *buff,PetscInt n)
 {
-  PetscInt  i,j,tmp = 0;
-  char       *ptr1,*ptr2 = (char*)&tmp;
-                                   
+  PetscInt i,j,tmp = 0;
+  char     *ptr1,*ptr2 = (char*)&tmp;
+
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff + j);
-    for (i=0; i<(PetscInt)sizeof(PetscInt); i++) {
-      ptr2[i] = ptr1[sizeof(PetscInt)-1-i];
-    }
-    for (i=0; i<(PetscInt)sizeof(PetscInt); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt)sizeof(PetscInt); i++) ptr2[i] = ptr1[sizeof(PetscInt)-1-i];
+    for (i=0; i<(PetscInt)sizeof(PetscInt); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapShort"
 /*
   PetscByteSwapShort - Swap bytes in a short
 */
 PetscErrorCode  PetscByteSwapShort(short *buff,PetscInt n)
 {
-  PetscInt   i,j;
-  short      tmp;
-  char       *ptr1,*ptr2 = (char*)&tmp;
+  PetscInt i,j;
+  short    tmp;
+  char     *ptr1,*ptr2 = (char*)&tmp;
 
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff + j);
-    for (i=0; i<(PetscInt) sizeof(short); i++) {
-      ptr2[i] = ptr1[sizeof(short)-1-i];
-    }
-    for (i=0; i<(PetscInt) sizeof(short); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt) sizeof(short); i++) ptr2[i] = ptr1[sizeof(short)-1-i];
+    for (i=0; i<(PetscInt) sizeof(short); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapScalar"
 /*
   PetscByteSwapScalar - Swap bytes in a double
@@ -131,17 +117,13 @@ PetscErrorCode  PetscByteSwapScalar(PetscScalar *buff,PetscInt n)
 #endif
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff1 + j);
-    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) {
-      ptr2[i] = ptr1[sizeof(PetscReal)-1-i];
-    }
-    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) ptr2[i] = ptr1[sizeof(PetscReal)-1-i];
+    for (i=0; i<(PetscInt) sizeof(PetscReal); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapDouble"
 /*
   PetscByteSwapDouble - Swap bytes in a double
@@ -155,17 +137,13 @@ PetscErrorCode  PetscByteSwapDouble(double *buff,PetscInt n)
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff1 + j);
-    for (i=0; i<(PetscInt) sizeof(double); i++) {
-      ptr2[i] = ptr1[sizeof(double)-1-i];
-    }
-    for (i=0; i<(PetscInt) sizeof(double); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt) sizeof(double); i++) ptr2[i] = ptr1[sizeof(double)-1-i];
+    for (i=0; i<(PetscInt) sizeof(double); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwapFloat"
 /*
   PetscByteSwapFloat - Swap bytes in a float
@@ -173,23 +151,19 @@ PetscErrorCode  PetscByteSwapDouble(double *buff,PetscInt n)
 PetscErrorCode PetscByteSwapFloat(float *buff,PetscInt n)
 {
   PetscInt i,j;
-  float   tmp,*buff1 = (float*)buff;
+  float    tmp,*buff1 = (float*)buff;
   char     *ptr1,*ptr2 = (char*)&tmp;
 
   PetscFunctionBegin;
   for (j=0; j<n; j++) {
     ptr1 = (char*)(buff1 + j);
-    for (i=0; i<(PetscInt) sizeof(float); i++) {
-      ptr2[i] = ptr1[sizeof(float)-1-i];
-    }
-    for (i=0; i<(PetscInt) sizeof(float); i++) {
-      ptr1[i] = ptr2[i];
-    }
+    for (i=0; i<(PetscInt) sizeof(float); i++) ptr2[i] = ptr1[sizeof(float)-1-i];
+    for (i=0; i<(PetscInt) sizeof(float); i++) ptr1[i] = ptr2[i];
   }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscByteSwap"
 PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
 {
@@ -197,17 +171,17 @@ PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
 
   PetscFunctionBegin;
   if      (pdtype == PETSC_INT)    {ierr = PetscByteSwapInt((PetscInt*)data,count);CHKERRQ(ierr);}
-  else if (pdtype == PETSC_ENUM)   {ierr = PetscByteSwapEnum((PetscEnum*)data,count);CHKERRQ(ierr);}        
+  else if (pdtype == PETSC_ENUM)   {ierr = PetscByteSwapEnum((PetscEnum*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_BOOL)   {ierr = PetscByteSwapBool((PetscBool*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_SCALAR) {ierr = PetscByteSwapScalar((PetscScalar*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_DOUBLE) {ierr = PetscByteSwapDouble((double*)data,count);CHKERRQ(ierr);}
-  else if (pdtype == PETSC_FLOAT) {ierr = PetscByteSwapFloat((float*)data,count);CHKERRQ(ierr);}
+  else if (pdtype == PETSC_FLOAT)  {ierr = PetscByteSwapFloat((float*)data,count);CHKERRQ(ierr);}
   else if (pdtype == PETSC_SHORT)  {ierr = PetscByteSwapShort((short*)data,count);CHKERRQ(ierr);}
   PetscFunctionReturn(0);
 }
 
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscBinaryRead"
 /*@
    PetscBinaryRead - Reads from a binary file.
@@ -216,7 +190,7 @@ PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
 
    Input Parameters:
 +  fd - the file
-.  n  - the number of items to read 
+.  n  - the number of items to read
 -  type - the type of items to read (PETSC_INT, PETSC_DOUBLE or PETSC_SCALAR)
 
    Output Parameters:
@@ -226,7 +200,7 @@ PetscErrorCode PetscByteSwap(void *data,PetscDataType pdtype,PetscInt count)
 
    Level: developer
 
-   Notes: 
+   Notes:
    PetscBinaryRead() uses byte swapping to work on all machines; the files
    are written to file ALWAYS using big-endian ordering. On small-endian machines the numbers
    are converted to the small-endian format when they are read in from the file.
@@ -245,14 +219,33 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
   int               wsize,err;
   size_t            m = (size_t) n,maxblock = 65536;
   char              *pp = (char*)p;
-#if !defined(PETSC_WORDS_BIGENDIAN)
-  PetscErrorCode    ierr;
-  void              *ptmp = p; 
+#if defined(PETSC_USE_REAL___FLOAT128)
+  PetscBool         readdouble = PETSC_FALSE;
+  double            *ppp;
 #endif
+#if !defined(PETSC_WORDS_BIGENDIAN) || defined(PETSC_USE_REAL___FLOAT128)
+  PetscErrorCode    ierr;
+#endif
+#if !defined(PETSC_WORDS_BIGENDIAN)
+  void              *ptmp = p;
+#endif
+  char              *fname = NULL;
 
   PetscFunctionBegin;
+  if (n < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Trying to write a negative amount of data %D",n);
   if (!n) PetscFunctionReturn(0);
 
+  if (type == PETSC_FUNCTION) {
+    m            = 64;
+    type         = PETSC_CHAR;
+    fname        = (char*) malloc(m*sizeof(char));
+    if (!fname) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Cannot allocate space for function name");
+    pp           = (char*)fname;
+#if !defined(PETSC_WORDS_BIGENDIAN)
+    ptmp         = (void*)fname;
+#endif
+  }
+
   if (type == PETSC_INT)          m *= sizeof(PetscInt);
   else if (type == PETSC_SCALAR)  m *= sizeof(PetscScalar);
   else if (type == PETSC_DOUBLE)  m *= sizeof(double);
@@ -263,24 +256,56 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
   else if (type == PETSC_BOOL)   m *= sizeof(PetscBool);
   else if (type == PETSC_BIT_LOGICAL) m  = PetscBTLength(m)*sizeof(char);
   else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown type");
-  
+
+#if defined(PETSC_USE_REAL___FLOAT128)
+  ierr = PetscOptionsGetBool(NULL,NULL,"-binary_read_double",&readdouble,NULL);CHKERRQ(ierr);
+  /* If using __float128 precision we still read in doubles from file */
+  if (type == PETSC_SCALAR && readdouble) {
+    m    = m/2;
+    ierr = PetscMalloc1(n,&ppp);CHKERRQ(ierr);
+    pp   = (char*)ppp;
+  }
+#endif
+
   while (m) {
     wsize = (m < maxblock) ? m : maxblock;
-    err = read(fd,pp,wsize);
+    err   = read(fd,pp,wsize);
     if (err < 0 && errno == EINTR) continue;
     if (!err && wsize > 0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_READ,"Read past end of file");
     if (err < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_READ,"Error reading from file, errno %d",errno);
     m  -= err;
     pp += err;
   }
+
+#if defined(PETSC_USE_REAL___FLOAT128)
+  if (type == PETSC_SCALAR && readdouble) {
+    PetscScalar *pv = (PetscScalar*) p;
+    PetscInt    i;
+#if !defined(PETSC_WORDS_BIGENDIAN)
+    ierr = PetscByteSwapDouble(ppp,n);CHKERRQ(ierr);
+#endif
+    for (i=0; i<n; i++) pv[i] = ppp[i];
+    ierr = PetscFree(ppp);CHKERRQ(ierr);
+    PetscFunctionReturn(0);
+  }
+#endif
+
 #if !defined(PETSC_WORDS_BIGENDIAN)
   ierr = PetscByteSwap(ptmp,type,n);CHKERRQ(ierr);
 #endif
 
+  if (type == PETSC_FUNCTION) {
+#if defined(PETSC_SERIALIZE_FUNCTIONS)
+    ierr = PetscDLSym(NULL,fname,(void**)p);CHKERRQ(ierr);
+#else
+    *(void**)p = NULL;
+#endif
+    free(fname);
+  }
   PetscFunctionReturn(0);
 }
 /* --------------------------------------------------------- */
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscBinaryWrite"
 /*@
    PetscBinaryWrite - Writes to a binary file.
@@ -296,7 +321,7 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
 
    Level: advanced
 
-   Notes: 
+   Notes:
    PetscBinaryWrite() uses byte swapping to work on all machines; the files
    are written using big-endian ordering to the file. On small-endian machines the numbers
    are converted to the big-endian format when they are written to disk.
@@ -304,22 +329,24 @@ PetscErrorCode  PetscBinaryRead(int fd,void *p,PetscInt n,PetscDataType type)
    file as 64 bit integers, this means they can only be read back in when the option --with-64bit-indices
    is used.
 
+   If running with __float128 precision the output is in __float128 unless one uses the -binary_read_double option
+
    The Buffer p should be read-write buffer, and not static data.
    This way, byte-swapping is done in-place, and then the buffer is
    written to the file.
-   
-   This routine restores the original contents of the buffer, after 
-   it is written to the file. This is done by byte-swapping in-place 
+
+   This routine restores the original contents of the buffer, after
+   it is written to the file. This is done by byte-swapping in-place
    the second time. If the flag istemp is set to PETSC_TRUE, the second
    byte-swapping operation is not done, thus saving some computation,
-   but the buffer corrupted is corrupted.
+   but the buffer is left corrupted.
 
    Because byte-swapping may be done on the values in data it cannot be declared const
 
    Concepts: files^writing binary
    Concepts: binary files^writing
 
-.seealso: PetscBinaryRead(), PetscBinaryOpen(), PetscBinaryClose(), PetscViewerBinaryGetDescriptor(), PetscBinarySynchronizedWrite(), 
+.seealso: PetscBinaryRead(), PetscBinaryOpen(), PetscBinaryClose(), PetscViewerBinaryGetDescriptor(), PetscBinarySynchronizedWrite(),
           PetscBinarySynchronizedRead(), PetscBinarySynchronizedSeek()
 @*/
 PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,PetscBool  istemp)
@@ -327,15 +354,58 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
   char           *pp = (char*)p;
   int            err,wsize;
   size_t         m = (size_t)n,maxblock=65536;
-#if !defined(PETSC_WORDS_BIGENDIAN)
   PetscErrorCode ierr;
-  void           *ptmp = p; 
+#if !defined(PETSC_WORDS_BIGENDIAN)
+  void           *ptmp = p;
+#endif
+  char           *fname = NULL;
+#if defined(PETSC_USE_REAL___FLOAT128)
+  PetscBool      writedouble = PETSC_FALSE;
+  double         *ppp;
+  PetscReal      *pv;
+  PetscInt       i;
 #endif
 
   PetscFunctionBegin;
   if (n < 0) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Trying to write a negative amount of data %D",n);
   if (!n) PetscFunctionReturn(0);
 
+  if (type == PETSC_FUNCTION) {
+#if defined(PETSC_SERIALIZE_FUNCTIONS)
+    const char *fnametmp;
+#endif
+    m     = 64;
+    fname = (char*)malloc(m*sizeof(char));
+    if (!fname) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Cannot allocate space for function name");
+#if defined(PETSC_SERIALIZE_FUNCTIONS)
+    if (n > 1) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_SUP,"Can only binary view a single function at a time");
+    ierr = PetscFPTFind(*(void**)p,&fnametmp);CHKERRQ(ierr);
+    ierr = PetscStrncpy(fname,fnametmp,m);CHKERRQ(ierr);
+#else
+    ierr = PetscStrncpy(fname,"",m);CHKERRQ(ierr);
+#endif
+    type = PETSC_CHAR;
+    pp   = (char*)fname;
+#if !defined(PETSC_WORDS_BIGENDIAN)
+    ptmp = (void*)fname;
+#endif
+  }
+
+#if defined(PETSC_USE_REAL___FLOAT128)
+  ierr = PetscOptionsGetBool(NULL,NULL,"-binary_write_double",&writedouble,NULL);CHKERRQ(ierr);
+  /* If using __float128 precision we still write in doubles to file */
+  if (type == PETSC_SCALAR && writedouble) {
+    m    = m/2;
+    ierr = PetscMalloc1(n,&ppp);CHKERRQ(ierr);
+    pv = (PetscReal*)pp;
+    for (i=0; i<n; i++) {
+      ppp[i] = (double) pv[i];
+    }
+    pp   = (char*)ppp;
+    ptmp = (char*)ppp;
+  }
+#endif
+
   if (type == PETSC_INT)          m *= sizeof(PetscInt);
   else if (type == PETSC_SCALAR)  m *= sizeof(PetscScalar);
   else if (type == PETSC_DOUBLE)  m *= sizeof(double);
@@ -353,10 +423,10 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
 
   while (m) {
     wsize = (m < maxblock) ? m : maxblock;
-    err = write(fd,pp,wsize);
+    err   = write(fd,pp,wsize);
     if (err < 0 && errno == EINTR) continue;
-    if (err != wsize) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_FILE_WRITE,"Error writing to file.");
-    m -= wsize;
+    if (err != wsize) SETERRQ3(PETSC_COMM_SELF,PETSC_ERR_FILE_WRITE,"Error writing to file total size %d err %d wsize %d",(int)n,(int)err,(int)wsize);
+    m  -= wsize;
     pp += wsize;
   }
 
@@ -365,11 +435,19 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
     ierr = PetscByteSwap(ptmp,type,n);CHKERRQ(ierr);
   }
 #endif
+  if (type == PETSC_FUNCTION) {
+    free(fname);
+  }
+#if defined(PETSC_USE_REAL___FLOAT128)
+  if (type == PETSC_SCALAR && writedouble) {
+    ierr = PetscFree(ppp);CHKERRQ(ierr);
+  }
+#endif   
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "PetscBinaryOpen" 
+#undef __FUNCT__
+#define __FUNCT__ "PetscBinaryOpen"
 /*@C
    PetscBinaryOpen - Opens a PETSc binary file.
 
@@ -391,47 +469,35 @@ PetscErrorCode  PetscBinaryWrite(int fd,void *p,PetscInt n,PetscDataType type,Pe
    big-endian format. This means the file can be accessed using PetscBinaryOpen() and
    PetscBinaryRead() and PetscBinaryWrite() on any machine.
 
-.seealso: PetscBinaryRead(), PetscBinaryWrite(), PetscFileMode, PetscViewerFileSetMode(), PetscViewerBinaryGetDescriptor(), 
+.seealso: PetscBinaryRead(), PetscBinaryWrite(), PetscFileMode, PetscViewerFileSetMode(), PetscViewerBinaryGetDescriptor(),
           PetscBinarySynchronizedWrite(), PetscBinarySynchronizedRead(), PetscBinarySynchronizedSeek()
 
 @*/
 PetscErrorCode  PetscBinaryOpen(const char name[],PetscFileMode mode,int *fd)
 {
   PetscFunctionBegin;
-#if defined(PETSC_HAVE_O_BINARY) 
+#if defined(PETSC_HAVE_O_BINARY)
   if (mode == FILE_MODE_WRITE) {
-    if ((*fd = open(name,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,0666)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot create file for writing: %s",name);
-    }
+    if ((*fd = open(name,O_WRONLY|O_CREAT|O_TRUNC|O_BINARY,0666)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot create file for writing: %s",name);
   } else if (mode == FILE_MODE_READ) {
-    if ((*fd = open(name,O_RDONLY|O_BINARY,0)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for reading: %s",name);
-    }
+    if ((*fd = open(name,O_RDONLY|O_BINARY,0)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for reading: %s",name);
   } else if (mode == FILE_MODE_APPEND) {
-    if ((*fd = open(name,O_WRONLY|O_BINARY,0)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for writing: %s",name);
-    }
+    if ((*fd = open(name,O_WRONLY|O_BINARY,0)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for writing: %s",name);
 #else
   if (mode == FILE_MODE_WRITE) {
-    if ((*fd = creat(name,0666)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot create file for writing: %s",name);
-    }
+    if ((*fd = creat(name,0666)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot create file for writing: %s",name);
   } else if (mode == FILE_MODE_READ) {
-    if ((*fd = open(name,O_RDONLY,0)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for reading: %s",name);
-    }
+    if ((*fd = open(name,O_RDONLY,0)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for reading: %s",name);
   }
   else if (mode == FILE_MODE_APPEND) {
-    if ((*fd = open(name,O_WRONLY,0)) == -1) {
-      SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for writing: %s",name);
-    }
+    if ((*fd = open(name,O_WRONLY,0)) == -1) SETERRQ1(PETSC_COMM_SELF,PETSC_ERR_FILE_OPEN,"Cannot open file for writing: %s",name);
 #endif
   } else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown file mode");
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "PetscBinaryClose" 
+#undef __FUNCT__
+#define __FUNCT__ "PetscBinaryClose"
 /*@
    PetscBinaryClose - Closes a PETSc binary file.
 
@@ -453,8 +519,8 @@ PetscErrorCode  PetscBinaryClose(int fd)
 }
 
 
-#undef __FUNCT__  
-#define __FUNCT__ "PetscBinarySeek" 
+#undef __FUNCT__
+#define __FUNCT__ "PetscBinarySeek"
 /*@
    PetscBinarySeek - Moves the file pointer on a PETSc binary file.
 
@@ -473,7 +539,7 @@ PetscErrorCode  PetscBinaryClose(int fd)
 
    Level: developer
 
-   Notes: 
+   Notes:
    Integers are stored on the file as 32 long, regardless of whether
    they are stored in the machine as 32 or 64, this means the same
    binary file may be read on any machine. Hence you CANNOT use sizeof()
@@ -490,15 +556,10 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
   int iwhence = 0;
 
   PetscFunctionBegin;
-  if (whence == PETSC_BINARY_SEEK_SET) {
-    iwhence = SEEK_SET;
-  } else if (whence == PETSC_BINARY_SEEK_CUR) {
-    iwhence = SEEK_CUR;
-  } else if (whence == PETSC_BINARY_SEEK_END) {
-    iwhence = SEEK_END;
-  } else {
-    SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown seek location");
-  }
+  if (whence == PETSC_BINARY_SEEK_SET) iwhence = SEEK_SET;
+  else if (whence == PETSC_BINARY_SEEK_CUR) iwhence = SEEK_CUR;
+  else if (whence == PETSC_BINARY_SEEK_END) iwhence = SEEK_END;
+  else SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_OUTOFRANGE,"Unknown seek location");
 #if defined(PETSC_HAVE_LSEEK)
   *offset = lseek(fd,off,iwhence);
 #elif defined(PETSC_HAVE__LSEEK)
@@ -509,7 +570,7 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscBinarySynchronizedRead"
 /*@C
    PetscBinarySynchronizedRead - Reads from a binary file.
@@ -517,22 +578,17 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
    Collective on MPI_Comm
 
    Input Parameters:
-+  comm - the MPI communicator 
++  comm - the MPI communicator
 .  fd - the file
-.  n  - the number of items to read 
+.  n  - the number of items to read
 -  type - the type of items to read (PETSC_INT, PETSC_DOUBLE or PETSC_SCALAR)
 
    Output Parameters:
 .  p - the buffer
 
-   Options Database Key:
-.   -binary_longints - indicates the file was generated on a Cray vector 
-         machine (not the T3E/D) and the ints are stored as 64 bit 
-         quantities, otherwise they are stored as 32 bit
-
    Level: developer
 
-   Notes: 
+   Notes:
    Does a PetscBinaryRead() followed by an MPI_Bcast()
 
    PetscBinarySynchronizedRead() uses byte swapping to work on all machines.
@@ -543,7 +599,7 @@ PetscErrorCode  PetscBinarySeek(int fd,off_t off,PetscBinarySeekType whence,off_
    Concepts: files^synchronized reading of binary files
    Concepts: binary files^reading, synchronized
 
-.seealso: PetscBinaryWrite(), PetscBinaryOpen(), PetscBinaryClose(), PetscBinaryRead(), PetscBinarySynchronizedWrite(), 
+.seealso: PetscBinaryWrite(), PetscBinaryOpen(), PetscBinaryClose(), PetscBinaryRead(), PetscBinarySynchronizedWrite(),
           PetscBinarySynchronizedSeek()
 @*/
 PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscInt n,PetscDataType type)
@@ -551,18 +607,38 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
   PetscErrorCode ierr;
   PetscMPIInt    rank;
   MPI_Datatype   mtype;
+  char           *fname = NULL;
+  void           *ptmp = NULL;
 
   PetscFunctionBegin;
+  if (type == PETSC_FUNCTION) {
+    n            = 64;
+    type         = PETSC_CHAR;
+    ptmp         = p;
+    fname        = (char*)malloc(n*sizeof(char));
+    if (!fname) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_MEM,"Cannot allocate space for function name");
+    p            = (void*)fname;
+  }
+
   ierr = MPI_Comm_rank(comm,&rank);CHKERRQ(ierr);
   if (!rank) {
     ierr = PetscBinaryRead(fd,p,n,type);CHKERRQ(ierr);
   }
   ierr = PetscDataTypeToMPIDataType(type,&mtype);CHKERRQ(ierr);
   ierr = MPI_Bcast(p,n,mtype,0,comm);CHKERRQ(ierr);
+
+  if (type == PETSC_FUNCTION) {
+#if defined(PETSC_SERIALIZE_FUNCTIONS)
+    ierr = PetscDLLibrarySym(PETSC_COMM_SELF,&PetscDLLibrariesLoaded,NULL,fname,(void**)ptmp);CHKERRQ(ierr);
+#else
+    *(void**)ptmp = NULL;
+#endif
+    free(fname);
+  }
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "PetscBinarySynchronizedWrite"
 /*@C
    PetscBinarySynchronizedWrite - writes to a binary file.
@@ -570,7 +646,7 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
    Collective on MPI_Comm
 
    Input Parameters:
-+  comm - the MPI communicator 
++  comm - the MPI communicator
 .  fd - the file
 .  n  - the number of items to write
 .  p - the buffer
@@ -579,7 +655,7 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
 
    Level: developer
 
-   Notes: 
+   Notes:
    Process 0 does a PetscBinaryWrite()
 
    PetscBinarySynchronizedWrite() uses byte swapping to work on all machines.
@@ -598,7 +674,7 @@ PetscErrorCode  PetscBinarySynchronizedRead(MPI_Comm comm,int fd,void *p,PetscIn
 .seealso: PetscBinaryWrite(), PetscBinaryOpen(), PetscBinaryClose(), PetscBinaryRead(), PetscBinarySynchronizedRead(),
           PetscBinarySynchronizedSeek()
 @*/
-PetscErrorCode  PetscBinarySynchronizedWrite(MPI_Comm comm,int fd,void *p,PetscInt n,PetscDataType type,PetscBool  istemp)
+PetscErrorCode  PetscBinarySynchronizedWrite(MPI_Comm comm,int fd,void *p,PetscInt n,PetscDataType type,PetscBool istemp)
 {
   PetscErrorCode ierr;
   PetscMPIInt    rank;
@@ -611,8 +687,8 @@ PetscErrorCode  PetscBinarySynchronizedWrite(MPI_Comm comm,int fd,void *p,PetscI
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
-#define __FUNCT__ "PetscBinarySynchronizedSeek" 
+#undef __FUNCT__
+#define __FUNCT__ "PetscBinarySynchronizedSeek"
 /*@C
    PetscBinarySynchronizedSeek - Moves the file pointer on a PETSc binary file.
 
@@ -630,14 +706,14 @@ PetscErrorCode  PetscBinarySynchronizedWrite(MPI_Comm comm,int fd,void *p,PetscI
 
    Level: developer
 
-   Notes: 
+   Notes:
    Integers are stored on the file as 32 long, regardless of whether
    they are stored in the machine as 32 or 64, this means the same
    binary file may be read on any machine. Hence you CANNOT use sizeof()
    to determine the offset or location.
 
    Concepts: binary files^seeking
-   Concepts: files^seeking in binary 
+   Concepts: files^seeking in binary
 
 .seealso: PetscBinaryRead(), PetscBinaryWrite(), PetscBinaryOpen(), PetscBinarySynchronizedWrite(), PetscBinarySynchronizedRead(),
           PetscBinarySynchronizedSeek()
@@ -659,7 +735,6 @@ PetscErrorCode  PetscBinarySynchronizedSeek(MPI_Comm comm,int fd,off_t off,Petsc
 #if !defined(PETSC_WORDS_BIGENDIAN)
 
 #if defined(PETSC_USE_PETSC_MPI_EXTERNAL32)
-EXTERN_C_BEGIN
 /*
       MPICH does not provide the external32 representation for MPI_File_set_view() so we need to provide the functions.
     These are set into MPI in PetscInitialize() via MPI_Register_datarep()
@@ -669,52 +744,51 @@ EXTERN_C_BEGIN
     The next three routines are not used because MPICH does not support their use
 
 */
-PetscMPIInt PetscDataRep_extent_fn(MPI_Datatype datatype,MPI_Aint *file_extent,void *extra_state) 
+PETSC_EXTERN PetscMPIInt PetscDataRep_extent_fn(MPI_Datatype datatype,MPI_Aint *file_extent,void *extra_state)
 {
   MPI_Aint    ub;
   PetscMPIInt ierr;
-  
+
   ierr = MPI_Type_get_extent(datatype,&ub,file_extent);
   return ierr;
 }
 
-PetscMPIInt PetscDataRep_read_conv_fn(void *userbuf, MPI_Datatype datatype,PetscMPIInt count,void *filebuf, MPI_Offset position,void *extra_state) 
+PETSC_EXTERN PetscMPIInt PetscDataRep_read_conv_fn(void *userbuf, MPI_Datatype datatype,PetscMPIInt count,void *filebuf, MPI_Offset position,void *extra_state)
 {
   PetscDataType pdtype;
   PetscMPIInt   ierr;
   size_t        dsize;
-  
+
   ierr = PetscMPIDataTypeToPetscDataType(datatype,&pdtype);CHKERRQ(ierr);
   ierr = PetscDataTypeGetSize(pdtype,&dsize);CHKERRQ(ierr);
 
   /* offset is given in units of MPI_Datatype */
-  userbuf = ((char *)userbuf) + dsize*position;
+  userbuf = ((char*)userbuf) + dsize*position;
 
   ierr = PetscMemcpy(userbuf,filebuf,count*dsize);CHKERRQ(ierr);
   ierr = PetscByteSwap(userbuf,pdtype,count);CHKERRQ(ierr);
   return ierr;
 }
 
-PetscMPIInt PetscDataRep_write_conv_fn(void *userbuf, MPI_Datatype datatype,PetscMPIInt count,void *filebuf, MPI_Offset position,void *extra_state) 
+PetscMPIInt PetscDataRep_write_conv_fn(void *userbuf, MPI_Datatype datatype,PetscMPIInt count,void *filebuf, MPI_Offset position,void *extra_state)
 {
   PetscDataType pdtype;
   PetscMPIInt   ierr;
   size_t        dsize;
-  
+
   ierr = PetscMPIDataTypeToPetscDataType(datatype,&pdtype);CHKERRQ(ierr);
   ierr = PetscDataTypeGetSize(pdtype,&dsize);CHKERRQ(ierr);
 
   /* offset is given in units of MPI_Datatype */
-  userbuf = ((char *)userbuf) + dsize*position;
+  userbuf = ((char*)userbuf) + dsize*position;
 
   ierr = PetscMemcpy(filebuf,userbuf,count*dsize);CHKERRQ(ierr);
-  ierr = PetscByteSwap(filebuf,pdtype,count);CHKERRQ(ierr);  
+  ierr = PetscByteSwap(filebuf,pdtype,count);CHKERRQ(ierr);
   return ierr;
 }
-EXTERN_C_END
 #endif
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MPIU_File_write_all"
 PetscErrorCode MPIU_File_write_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Datatype dtype,MPI_Status *status)
 {
@@ -723,13 +797,13 @@ PetscErrorCode MPIU_File_write_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Da
 
   PetscFunctionBegin;
   ierr = PetscMPIDataTypeToPetscDataType(dtype,&pdtype);CHKERRQ(ierr);
-  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);  
+  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);
   ierr = MPI_File_write_all(fd,data,cnt,dtype,status);CHKERRQ(ierr);
-  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);  
+  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
-#undef __FUNCT__  
+#undef __FUNCT__
 #define __FUNCT__ "MPIU_File_read_all"
 PetscErrorCode MPIU_File_read_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Datatype dtype,MPI_Status *status)
 {
@@ -739,7 +813,7 @@ PetscErrorCode MPIU_File_read_all(MPI_File fd,void *data,PetscMPIInt cnt,MPI_Dat
   PetscFunctionBegin;
   ierr = PetscMPIDataTypeToPetscDataType(dtype,&pdtype);CHKERRQ(ierr);
   ierr = MPI_File_read_all(fd,data,cnt,dtype,status);CHKERRQ(ierr);
-  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);  
+  ierr = PetscByteSwap(data,pdtype,cnt);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 #endif
