@@ -2,49 +2,63 @@
       This provides a few of the MPI-uni functions that cannot be implemented
     with C macros
 */
-#include "mpiuni/mpi.h"
+#include <petscsys.h>
 #if !defined(__MPIUNI_H)
 #error "Wrong mpi.h included! require mpi.h from MPIUNI"
 #endif
 #if !defined(PETSC_STDCALL)
 #define PETSC_STDCALL
 #endif
-#include <stdio.h>
-#if defined(PETSC_HAVE_STDLIB_H)
-#include <stdlib.h>
-#endif
 
 #define MPI_SUCCESS 0
 #define MPI_FAILURE 1
-void    *MPIUNI_TMP        = 0;
-int     MPIUNI_DATASIZE[10] = {sizeof(int),sizeof(float),sizeof(double),2*sizeof(double),sizeof(char),2*sizeof(int),4*sizeof(double),4,8,2*sizeof(double)};
+
+void *MPIUNI_TMP         = 0;
+int  MPIUNI_DATASIZE[10] = {sizeof(int),sizeof(float),sizeof(double),2*sizeof(double),sizeof(char),2*sizeof(int),4*sizeof(double),4,8,2*sizeof(double)};
 /*
-       With MPI Uni there is only one communicator, which is called 1.
+       With MPI Uni there are exactly four distinct communicators:
+    MPI_COMM_SELF, MPI_COMM_WORLD, and a MPI_Comm_dup() of each of these (duplicates of duplicates return the same communictor)
+
+    MPI_COMM_SELF and MPI_COMM_WORLD are MPI_Comm_free() in MPI_Finalize() but in general with PETSc,
+     the other communicators are freed once the last PETSc object is freed (before MPI_Finalize()).
+
 */
 #define MAX_ATTR 128
+#define MAX_COMM 128
+
+static int MaxComm = 2;
+
+typedef struct {
+  void *attribute_val;
+  int  active;
+} MPI_Attr;
 
 typedef struct {
   void                *extra_state;
-  void                *attribute_val;
-  int                 active;
   MPI_Delete_function *del;
-} MPI_Attr;
+} MPI_Attr_keyval;
 
-static MPI_Attr attr[MAX_ATTR];
-static int      num_attr = 1,mpi_tag_ub = 100000000;
+static MPI_Attr_keyval attr_keyval[MAX_ATTR];
+static MPI_Attr        attr[MAX_COMM][MAX_ATTR];
+static int             comm_active[MAX_COMM];
+static int             num_attr = 1,mpi_tag_ub = 100000000;
+static void*           MPIUNIF_mpi_in_place = 0;
 
 #if defined(__cplusplus)
 extern "C" {
 #endif
 
-/* 
+/*
    To avoid problems with prototypes to the system memcpy() it is duplicated here
 */
-int MPIUNI_Memcpy(void *a,const void* b,int n) {
+int MPIUNI_Memcpy(void *a,const void *b,int n)
+{
   int  i;
   char *aa= (char*)a;
   char *bb= (char*)b;
 
+  if (a == MPI_IN_PLACE || a == MPIUNIF_mpi_in_place) return 0;
+  if (b == MPI_IN_PLACE || b == MPIUNIF_mpi_in_place) return 0;
   for (i=0; i<n; i++) aa[i] = bb[i];
   return 0;
 }
@@ -54,8 +68,10 @@ int MPIUNI_Memcpy(void *a,const void* b,int n) {
 */
 static int Keyval_setup(void)
 {
-  attr[0].active        = 1;
-  attr[0].attribute_val = &mpi_tag_ub;
+  attr[MPI_COMM_WORLD-1][0].active        = 1;
+  attr[MPI_COMM_WORLD-1][0].attribute_val = &mpi_tag_ub;
+  attr[MPI_COMM_SELF-1][0].active         = 1;
+  attr[MPI_COMM_SELF-1][0].attribute_val  = &mpi_tag_ub;
   return 0;
 }
 
@@ -63,54 +79,81 @@ int MPI_Keyval_create(MPI_Copy_function *copy_fn,MPI_Delete_function *delete_fn,
 {
   if (num_attr >= MAX_ATTR) MPI_Abort(MPI_COMM_WORLD,1);
 
-  attr[num_attr].extra_state = extra_state;
-  attr[num_attr].del         = delete_fn;
-  *keyval                    = num_attr++;
+  attr_keyval[num_attr].extra_state = extra_state;
+  attr_keyval[num_attr].del         = delete_fn;
+  *keyval                           = num_attr++;
   return 0;
 }
 
 int MPI_Keyval_free(int *keyval)
 {
-  attr[*keyval].active = 0;
+  attr_keyval[*keyval].extra_state = 0;
+  attr_keyval[*keyval].del         = 0;
+
+  *keyval = 0;
   return MPI_SUCCESS;
 }
 
 int MPI_Attr_put(MPI_Comm comm,int keyval,void *attribute_val)
 {
-  attr[keyval].active        = 1;
-  attr[keyval].attribute_val = attribute_val;
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  attr[comm-1][keyval].active        = 1;
+  attr[comm-1][keyval].attribute_val = attribute_val;
   return MPI_SUCCESS;
 }
-  
+
 int MPI_Attr_delete(MPI_Comm comm,int keyval)
 {
-  if (attr[keyval].active && attr[keyval].del) {
-    (*(attr[keyval].del))(comm,keyval,attr[keyval].attribute_val,attr[keyval].extra_state);
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  if (attr[comm-1][keyval].active && attr_keyval[keyval].del) {
+    void *save_attribute_val = attr[comm-1][keyval].attribute_val;
+    attr[comm-1][keyval].active        = 0;
+    attr[comm-1][keyval].attribute_val = 0;
+    (*(attr_keyval[keyval].del))(comm,keyval,save_attribute_val,attr_keyval[keyval].extra_state);
   }
-  attr[keyval].active        = 0;
-  attr[keyval].attribute_val = 0;
   return MPI_SUCCESS;
 }
 
 int MPI_Attr_get(MPI_Comm comm,int keyval,void *attribute_val,int *flag)
 {
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
   if (!keyval) Keyval_setup();
-  *flag                   = attr[keyval].active;
-  *(void **)attribute_val = attr[keyval].attribute_val;
+  *flag                  = attr[comm-1][keyval].active;
+  *(void**)attribute_val = attr[comm-1][keyval].attribute_val;
   return MPI_SUCCESS;
 }
 
 int MPI_Comm_create(MPI_Comm comm,MPI_Group group,MPI_Comm *newcomm)
 {
-  *newcomm =  comm;
+  int j;
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  for (j=3; j<MaxComm; j++) {
+    if (!comm_active[j-1]) {
+      comm_active[j-1] = 1;
+      *newcomm = j;
+      return MPI_SUCCESS;
+    }
+  }
+  if (MaxComm > MAX_COMM) return MPI_FAILURE;
+  *newcomm =  MaxComm++;
+  comm_active[*newcomm-1] = 1;
   return MPI_SUCCESS;
 }
 
-static int dups = 0;
 int MPI_Comm_dup(MPI_Comm comm,MPI_Comm *out)
 {
-  *out = comm;
-  dups++;
+  int j;
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  for (j=3; j<MaxComm; j++) {
+    if (!comm_active[j-1]) {
+      comm_active[j-1] = 1;
+      *out = j;
+      return MPI_SUCCESS;
+    }
+  }
+  if (MaxComm > MAX_COMM) return MPI_FAILURE;
+  *out = MaxComm++;
+  comm_active[*out-1] = 1;
   return 0;
 }
 
@@ -118,24 +161,39 @@ int MPI_Comm_free(MPI_Comm *comm)
 {
   int i;
 
-  if (--dups) return MPI_SUCCESS;
+  if (*comm-1 < 0 || *comm-1 > MaxComm) return 1;
   for (i=0; i<num_attr; i++) {
-    if (attr[i].active && attr[i].del) {
-      (*attr[i].del)(*comm,i,attr[i].attribute_val,attr[i].extra_state);
-    }
-    attr[i].active = 0;
+    if (attr[*comm-1][i].active && attr_keyval[i].del) (*attr_keyval[i].del)(*comm,i,attr[*comm-1][i].attribute_val,attr_keyval[i].extra_state);
+    attr[*comm-1][i].active        = 0;
+    attr[*comm-1][i].attribute_val = 0;
   }
+  if (*comm >= 3) comm_active[*comm-1] = 0;
+  *comm = 0;
   return MPI_SUCCESS;
 }
 
-int MPI_Abort(MPI_Comm comm,int errorcode) 
+int MPI_Comm_size(MPI_Comm comm, int *size)
+{
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  *size=1;
+  return MPI_SUCCESS;
+}
+
+int MPI_Comm_rank(MPI_Comm comm, int *rank)
+{
+  if (comm-1 < 0 || comm-1 > MaxComm) return 1;
+  *rank=0;
+  return MPI_SUCCESS;
+}
+
+int MPI_Abort(MPI_Comm comm,int errorcode)
 {
   abort();
   return MPI_SUCCESS;
 }
 
 /* --------------------------------------------------------------------------*/
-  
+
 static int MPI_was_initialized = 0;
 static int MPI_was_finalized   = 0;
 
@@ -149,8 +207,13 @@ int MPI_Init(int *argc, char ***argv)
 
 int MPI_Finalize(void)
 {
+  MPI_Comm comm;
   if (MPI_was_finalized) return 1;
   if (!MPI_was_initialized) return 1;
+  comm = MPI_COMM_WORLD;
+  MPI_Comm_free(&comm);
+  comm = MPI_COMM_SELF;
+  MPI_Comm_free(&comm);
   MPI_was_finalized = 1;
   return 0;
 }
@@ -170,6 +233,8 @@ int MPI_Finalized(int *flag)
 /* -------------------     Fortran versions of several routines ------------------ */
 
 #if defined(PETSC_HAVE_FORTRAN_CAPS)
+#define mpiunisetcommonblock_ MPIUNISETCOMMONBLOCK
+#define mpiunisetfortranbasepointers_  MPIUNISETFORTRANBASEPOINTERS
 #define mpi_init_             MPI_INIT
 #define mpi_finalize_         MPI_FINALIZE
 #define mpi_comm_size_        MPI_COMM_SIZE
@@ -216,6 +281,8 @@ int MPI_Finalized(int *flag)
 #define mpi_comm_group_       MPI_COMM_GROUP
 #define mpi_exscan_           MPI_EXSCAN
 #elif !defined(PETSC_HAVE_FORTRAN_UNDERSCORE)
+#define mpiunisetcommonblock_ mpiunisetcommonblock
+#define mpiunisetfortranbasepointers_  mpiunisetfortranbasepointers
 #define mpi_init_             mpi_init
 #define mpi_finalize_         mpi_finalize
 #define mpi_comm_size_        mpi_comm_size
@@ -311,176 +378,174 @@ int MPI_Finalized(int *flag)
 #define mpi_exscan_           mpi_exscan__
 #endif
 
-
 /* Do not build fortran interface if MPI namespace colision is to be avoided */
-#if !defined(MPIUNI_AVOID_MPI_NAMESPACE)
+#if defined(MPIUNI_FORTRAN_BINDING)
 
-void PETSC_STDCALL  mpi_init_(int *ierr)
+PETSC_EXTERN void PETSC_STDCALL mpiunisetcommonblock_(void);
+
+PETSC_EXTERN void PETSC_STDCALL mpiunisetfortranbasepointers_(void *f_mpi_in_place)
 {
+  MPIUNIF_mpi_in_place   = f_mpi_in_place;
+}
+
+PETSC_EXTERN void PETSC_STDCALL   mpi_init_(int *ierr)
+{
+  mpiunisetcommonblock_();
   *ierr = MPI_Init((int*)0, (char***)0);
 }
 
-void PETSC_STDCALL  mpi_finalize_(int *ierr)
+PETSC_EXTERN void PETSC_STDCALL   mpi_finalize_(int *ierr)
 {
   *ierr = MPI_Finalize();
 }
 
-void PETSC_STDCALL mpi_comm_size_(MPI_Comm *comm,int *size,int *ierr) 
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_size_(MPI_Comm *comm,int *size,int *ierr)
 {
   *size = 1;
   *ierr = 0;
 }
 
-void PETSC_STDCALL mpi_comm_rank_(MPI_Comm *comm,int *rank,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_rank_(MPI_Comm *comm,int *rank,int *ierr)
 {
-  *rank=0;
-  *ierr=MPI_SUCCESS;
+  *rank = 0;
+  *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_comm_split_(MPI_Comm *comm,int *color,int *key, MPI_Comm *newcomm, int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_split_(MPI_Comm *comm,int *color,int *key, MPI_Comm *newcomm, int *ierr)
 {
   *newcomm = *comm;
-  *ierr=MPI_SUCCESS;
+  *ierr    = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_abort_(MPI_Comm *comm,int *errorcode,int *ierr) 
+PETSC_EXTERN void PETSC_STDCALL  mpi_abort_(MPI_Comm *comm,int *errorcode,int *ierr)
 {
   abort();
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_reduce_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *root,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_reduce_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *root,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*count)*MPIUNI_DATASIZE[*datatype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPI_Reduce(sendbuf,recvbuf,*count,*datatype,*op,*root,*comm);
 }
 
-void PETSC_STDCALL mpi_allreduce_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *comm,int *ierr) 
+PETSC_EXTERN void PETSC_STDCALL  mpi_allreduce_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*count)*MPIUNI_DATASIZE[*datatype]);
-  *ierr = MPI_SUCCESS;
-} 
+  *ierr = MPI_Allreduce(sendbuf,recvbuf,*count,*datatype,*op,*comm);
+}
 
-void PETSC_STDCALL mpi_barrier_(MPI_Comm *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_barrier_(MPI_Comm *comm,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_bcast_(void *buf,int *count,int *datatype,int *root,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_bcast_(void *buf,int *count,int *datatype,int *root,int *comm,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-
-void PETSC_STDCALL mpi_gather_(void *sendbuf,int *scount,int *sdatatype, void* recvbuf, int* rcount, int* rdatatype, int *root,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_gather_(void *sendbuf,int *scount,int *sdatatype, void *recvbuf, int *rcount, int *rdatatype, int *root,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*scount)*MPIUNI_DATASIZE[*sdatatype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPI_Gather(sendbuf,*scount,*sdatatype,recvbuf,rcount,rdatatype,*root,*comm);
 }
 
-void PETSC_STDCALL mpi_allgather_(void *sendbuf,int *scount,int *sdatatype, void* recvbuf, int* rcount, int* rdatatype,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_allgather_(void *sendbuf,int *scount,int *sdatatype, void *recvbuf, int *rcount, int *rdatatype,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*scount)*MPIUNI_DATASIZE[*sdatatype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPI_Allgather(sendbuf,*scount,*sdatatype,recvbuf,rcount,rdatatype,*comm);
 }
 
-void PETSC_STDCALL mpi_scan_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_scan_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*count)*MPIUNI_DATASIZE[*datatype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPIUNI_Memcpy(recvbuf,sendbuf,(*count)*MPIUNI_DATASIZE[*datatype]);
 }
 
-void PETSC_STDCALL mpi_send_(void*buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *ierr )
+PETSC_EXTERN void PETSC_STDCALL  mpi_send_(void *buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_recv_(void*buf,int *count,int *datatype,int *source,int *tag,int *comm,int status,int *ierr )
+PETSC_EXTERN void PETSC_STDCALL  mpi_recv_(void *buf,int *count,int *datatype,int *source,int *tag,int *comm,int status,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_reduce_scatter_(void*sendbuf,void*recvbuf,int *recvcounts,int *datatype,int *op,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_reduce_scatter_(void *sendbuf,void *recvbuf,int *recvcounts,int *datatype,int *op,int *comm,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_irecv_(void*buf,int *count, int *datatype, int *source, int *tag, int *comm, int *request, int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_irecv_(void *buf,int *count, int *datatype, int *source, int *tag, int *comm, int *request, int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_isend_(void*buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *request, int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_isend_(void *buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *request, int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_sendrecv_(void*sendbuf,int *sendcount,int *sendtype,int *dest,int *sendtag,void*recvbuf,int *recvcount,int *recvtype,int *source,int *recvtag,int *comm,int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_sendrecv_(void *sendbuf,int *sendcount,int *sendtype,int *dest,int *sendtag,void *recvbuf,int *recvcount,int *recvtype,int *source,int *recvtag,int *comm,int *status,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*sendcount)*MPIUNI_DATASIZE[*sendtype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPIUNI_Memcpy(recvbuf,sendbuf,(*sendcount)*MPIUNI_DATASIZE[*sendtype]);
 }
 
-void PETSC_STDCALL mpi_test_(int *request,int *flag,int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_test_(int *request,int *flag,int *status,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_waitall_(int *count,int *array_of_requests,int *array_of_statuses,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_waitall_(int *count,int *array_of_requests,int *array_of_statuses,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-  void PETSC_STDCALL mpi_waitany_(int *count,int *array_of_requests,int * index, int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_waitany_(int *count,int *array_of_requests,int * index, int *status,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_allgatherv_(void*sendbuf,int *sendcount,int *sendtype,void*recvbuf,int *recvcounts,int *displs,int *recvtype,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_allgatherv_(void *sendbuf,int *sendcount,int *sendtype,void *recvbuf,int *recvcounts,int *displs,int *recvtype,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*sendcount)*MPIUNI_DATASIZE[*sendtype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPI_Allgatherv(sendbuf,*sendcount,*sendtype,recvbuf,recvcounts,displs,*recvtype,*comm);
 }
 
-void PETSC_STDCALL mpi_alltoallv_(void*sendbuf,int *sendcounts,int *sdispls,int *sendtype,void*recvbuf,int *recvcounts,int *rdispls,int *recvtype,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_alltoallv_(void *sendbuf,int *sendcounts,int *sdispls,int *sendtype,void *recvbuf,int *recvcounts,int *rdispls,int *recvtype,int *comm,int *ierr)
 {
-  MPIUNI_Memcpy(recvbuf,sendbuf,(*sendcounts)*MPIUNI_DATASIZE[*sendtype]);
-  *ierr = MPI_SUCCESS;
+  *ierr = MPI_Alltoallv(sendbuf,sendcounts,sdispls,*sendtype,recvbuf,recvcounts,rdispls,*recvtype,*comm);
 }
 
-void PETSC_STDCALL mpi_comm_create_(int *comm,int *group,int *newcomm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_create_(int *comm,int *group,int *newcomm,int *ierr)
 {
   *newcomm =  *comm;
-  *ierr = MPI_SUCCESS;
+  *ierr    = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_address_(void*location,MPIUNI_INTPTR *address,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_address_(void *location,MPIUNI_INTPTR *address,int *ierr)
 {
   *address =  (MPIUNI_INTPTR) location;
-  *ierr = MPI_SUCCESS;
+  *ierr    = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_pack_(void*inbuf,int *incount,int *datatype,void*outbuf,int *outsize,int *position,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_pack_(void *inbuf,int *incount,int *datatype,void *outbuf,int *outsize,int *position,int *comm,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_unpack_(void*inbuf,int *insize,int *position,void*outbuf,int *outcount,int *datatype,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_unpack_(void *inbuf,int *insize,int *position,void *outbuf,int *outcount,int *datatype,int *comm,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_pack_size_(int *incount,int *datatype,int *comm,int *size,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_pack_size_(int *incount,int *datatype,int *comm,int *size,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_type_struct_(int *count,int *array_of_blocklengths,int * array_of_displaments,int *array_of_types,int *newtype,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_type_struct_(int *count,int *array_of_blocklengths,int * array_of_displaments,int *array_of_types,int *newtype,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_type_commit_(int *datatype,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_type_commit_(int *datatype,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
@@ -490,23 +555,23 @@ double PETSC_STDCALL mpi_wtime_(void)
   return 0.0;
 }
 
-void PETSC_STDCALL mpi_cancel_(int *request,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_cancel_(int *request,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_comm_dup_(int *comm,int *out,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_dup_(int *comm,int *out,int *ierr)
 {
-  *out = *comm;
+  *out  = *comm;
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_comm_free_(int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_free_(int *comm,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_get_count_(int *status,int *datatype,int *count,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_get_count_(int *status,int *datatype,int *count,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
@@ -520,50 +585,50 @@ void PETSC_STDCALL mpi_get_count_(int *status,int *datatype,int *count,int *ierr
 #define PETSC_END_LEN(len)   ,int len
 #endif
 
-void PETSC_STDCALL mpi_get_processor_name_(char *name PETSC_MIXED_LEN(len),int *result_len,int *ierr PETSC_END_LEN(len))
+PETSC_EXTERN void PETSC_STDCALL  mpi_get_processor_name_(char *name PETSC_MIXED_LEN(len),int *result_len,int *ierr PETSC_END_LEN(len))
 {
   MPIUNI_Memcpy(name,"localhost",9*sizeof(char));
   *result_len = 9;
-  *ierr = MPI_SUCCESS;
+  *ierr       = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_initialized_(int *flag,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_initialized_(int *flag,int *ierr)
 {
   *flag = MPI_was_initialized;
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_iprobe_(int *source,int *tag,int *comm,int *glag,int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_iprobe_(int *source,int *tag,int *comm,int *glag,int *status,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_probe_(int *source,int *tag,int *comm,int *flag,int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_probe_(int *source,int *tag,int *comm,int *flag,int *status,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_request_free_(int *request,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_request_free_(int *request,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_ssend_(void*buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_ssend_(void *buf,int *count,int *datatype,int *dest,int *tag,int *comm,int *ierr)
 {
   *ierr = MPI_Abort(MPI_COMM_WORLD,0);
 }
 
-void PETSC_STDCALL mpi_wait_(int *request,int *status,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_wait_(int *request,int *status,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_comm_group_(int*comm,int*group,int *ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_comm_group_(int *comm,int *group,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
 
-void PETSC_STDCALL mpi_exscan_(void*sendbuf,void*recvbuf,int*count,int*datatype,int*op,int*comm,int*ierr)
+PETSC_EXTERN void PETSC_STDCALL  mpi_exscan_(void *sendbuf,void *recvbuf,int *count,int *datatype,int *op,int *comm,int *ierr)
 {
   *ierr = MPI_SUCCESS;
 }
